/**
 * Copyright (c) 2000-2011 Liferay, Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 */

package com.liferay.webform.util;

import com.liferay.counter.service.CounterLocalServiceUtil;
import com.liferay.mozilla.javascript.Context;
import com.liferay.mozilla.javascript.Scriptable;
import com.liferay.mozilla.javascript.ScriptableObject;
import com.liferay.portal.kernel.exception.PortalException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.GetterUtil;
import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.kernel.util.Validator;
import com.liferay.portlet.expando.NoSuchTableException;
import com.liferay.portlet.expando.model.ExpandoColumnConstants;
import com.liferay.portlet.expando.model.ExpandoTable;
import com.liferay.portlet.expando.service.ExpandoColumnLocalServiceUtil;
import com.liferay.portlet.expando.service.ExpandoRowLocalServiceUtil;
import com.liferay.portlet.expando.service.ExpandoTableLocalServiceUtil;
import com.liferay.util.portlet.PortletProps;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.portlet.PortletPreferences;

/**
 * @author Daniel Weisser
 * @author Jorge Ferrer
 * @author Alberto Montero
 * @author Julio Camarero
 * @author Brian Wing Shun Chan
 */
public class WebFormUtil {

	public static final boolean VALIDATION_SCRIPT_ENABLED =
		GetterUtil.getBoolean(PortletProps.get("validation.script.enabled"));

	public static ExpandoTable addTable(long companyId, String tableName)
		throws PortalException, SystemException {

		try {
			ExpandoTableLocalServiceUtil.deleteTable(
				companyId, WebFormUtil.class.getName(), tableName);
		}
		catch (NoSuchTableException nste) {
		}

		return ExpandoTableLocalServiceUtil.addTable(
			companyId, WebFormUtil.class.getName(), tableName);
	}

	public static ExpandoTable checkTable(
			long companyId, String tableName, PortletPreferences preferences)
		throws Exception {

		ExpandoTable expandoTable = null;

		try {
			expandoTable = ExpandoTableLocalServiceUtil.getTable(
				companyId, WebFormUtil.class.getName(), tableName);
		}
		catch (NoSuchTableException nste) {
			expandoTable = addTable(companyId, tableName);

			int i = 1;

			String fieldLabel = preferences.getValue(
				"fieldLabel" + i, StringPool.BLANK);

			while ((i == 1) || (Validator.isNotNull(fieldLabel))) {
				ExpandoColumnLocalServiceUtil.addColumn(
					expandoTable.getTableId(), fieldLabel,
					ExpandoColumnConstants.STRING);

				i++;

				fieldLabel = preferences.getValue(
					"fieldLabel" + i, StringPool.BLANK);
			}
		}

		return expandoTable;
	}

	public static String getNewDatabaseTableName(String portletId)
		throws SystemException {

		long formId = CounterLocalServiceUtil.increment(
			WebFormUtil.class.getName());

		return portletId + StringPool.UNDERLINE + formId;
	}

	public static int getTableRowsCount(long companyId, String tableName)
		throws SystemException {

		return ExpandoRowLocalServiceUtil.getRowsCount(
			companyId, WebFormUtil.class.getName(), tableName);
	}

	public static String[] split(String s) {
		return split(s, StringPool.COMMA);
	}

	public static String[] split(String s, String delimiter) {
		if (s == null || delimiter == null) {
			return new String[0];
		}

		s = s.trim();

		if (!s.endsWith(delimiter)) {
			StringBuilder sb = new StringBuilder();

			sb.append(s);
			sb.append(delimiter);

			s = sb.toString();
		}

		if (s.equals(delimiter)) {
			return new String[0];
		}

		List<String> nodeValues = new ArrayList<String>();

		if (delimiter.equals("\n") || delimiter.equals("\r")) {
			try {
				BufferedReader br = new BufferedReader(new StringReader(s));

				String line = null;

				while ((line = br.readLine()) != null) {
					nodeValues.add(line);
				}

				br.close();
			}
			catch (IOException ioe) {
				ioe.printStackTrace();
			}
		}
		else {
			int offset = 0;
			int pos = s.indexOf(delimiter, offset);

			while (pos != -1) {
				nodeValues.add(new String(s.substring(offset, pos)));

				offset = pos + delimiter.length();
				pos = s.indexOf(delimiter, offset);
			}
		}

		return nodeValues.toArray(new String[nodeValues.size()]);
	}

	public static boolean validate(
			String currentFieldValue, Map<String,String> fieldsMap,
			String validationScript)
		throws Exception {

		boolean validationResult = false;

		Context context = Context.enter();

		StringBuilder sb = new StringBuilder();

		sb.append("currentFieldValue = String('" + currentFieldValue + "');\n");

		sb.append("var fieldsMap = {};\n");

		for (String key : fieldsMap.keySet()) {
			sb.append("fieldsMap['");
			sb.append(key);
			sb.append("'] = '");
			sb.append(fieldsMap.get(key));
			sb.append("';\n");
		}

		sb.append("function validation(currentFieldValue, fieldsMap) {\n");
		sb.append(validationScript);
		sb.append("};\n");
		sb.append("internalValidationResult = ");
		sb.append("validation(currentFieldValue, fieldsMap);");

		String script = sb.toString();

		try {
			Scriptable scope = context.initStandardObjects();

			Object jsFieldsMap = Context.javaToJS(fieldsMap, scope);

			ScriptableObject.putProperty(scope, "jsFieldsMap", jsFieldsMap);

			context.evaluateString(scope, script, "Validation Script", 1, null);

			Object obj = ScriptableObject.getProperty(
				scope, "internalValidationResult");

			if (obj instanceof Boolean) {
				validationResult = ((Boolean)obj).booleanValue();
			}
			else {
				throw new Exception("The script must return a boolean value");
			}
		}
		catch (Exception e) {
			String msg =
				"The following script has execution errors:\n" +
					validationScript + "\n" + e.getMessage();

			_log.error(msg);

			throw new Exception(msg, e);
		}
		finally {
			Context.exit();
		}

		return validationResult;
	}

	private static Log _log = LogFactoryUtil.getLog(WebFormUtil.class);

}